#![warn(missing_docs)]
//! A compliant wat compiler
//!
//! # Refs
//! https://webassembly.github.io/spec/core/
//! https://blog.ttulka.com/learning-webassembly-2-wasm-binary-format

use log::*;
//
pub const WASM_MAGIC: &[u8; 4] = b"\0asm";
pub const WASM_VERSION: &[u8; 4] = b"1000";
#[non_exhaustive]
pub enum WasmTypes {
    /// https://webassembly.github.io/spec/core/syntax/types.html#number-types
    /// thirty two bit numbers both signed and unsigned
    Number32(u32),
    /// sixty four bit numbers both signed and unsigned
    Number64(u64),
    Float32(f32),
    Float64(f64),

    /// https://webassembly.github.io/spec/core/syntax/types.html#vector-types
    Vector128,
}

#[non_exhaustive]
pub struct FunctionSignature {
    pub parameters: Vec<WasmTypes>,
    pub return_count: Vec<WasmTypes>,
}

pub enum SectionType {
    Type,
    Function(FunctionSignature),
    Export,
}

pub struct Section {
    stype: SectionType,
    size: u64,
}

pub struct Program {
    pub sections: Vec<Section>,
}

impl Program {
    pub fn to_binary(&self) -> Vec<u8> {
        let mut bin: Vec<u8> = vec![];
        for byte in WASM_MAGIC {
            bin.push(*byte);
        }
        trace!("Added WASM magic value: {:?}", WASM_MAGIC);

        for byte in WASM_VERSION {
            bin.push(*byte);
        }
        trace!("Added WASM version value: {:?}", WASM_VERSION);

        bin
    }
}

fn main() {
    let program = Program {
        // NOTE: If wasm ever gets a v2 this should probably be handled properly
        sections: vec![],
    };

    let pbin = program.to_binary();
    println!("{:?}", pbin);
}
